SystemOrganization addCategory: #MorphExamplesAndDemos!

TextMorph subclass: #ClipboardMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!ClipboardMorph commentStamp: 'StephaneDucasse 8/20/2009 16:02' prior: 0!
A morph that always displays the current contents of the text clipboard.

	ClipboardMorph new openInWorld!

!ClipboardMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 01:21'!
descriptionForPartsBin
	^ self partName:	'Clipboard'
		categories:		#('Useful')
		documentation:	'This object will always show whatever is on the text clipboard'! !

!ClipboardMorph methodsFor: 'initialization' stamp: 'StephaneDucasse 8/20/2009 16:00'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	
	^ Color
		r: 1.0
		g: 0.355
		b: 0.452! !

!ClipboardMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:36'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 6! !

!ClipboardMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:24'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color blue! !

!ClipboardMorph methodsFor: 'parts bin' stamp: 'StephaneDucasse 8/20/2009 16:01'!
initialize
	super initialize.
	self extent: 250 @ 200.
	self
		backgroundColor: (Color
				r: 0.484
				g: 1.0
				b: 0.484).
	self setBalloonText: 'This shows the current contents of the text clipboard'.
	self newContents: Clipboard clipboardText! !

!ClipboardMorph methodsFor: 'stepping and presenter' stamp: 'sw 6/27/2001 14:15'!
step
	self newContents: Clipboard clipboardText! !

!ClipboardMorph methodsFor: 'testing' stamp: 'sw 6/27/2001 14:18'!
stepTime
	"Answer the interval between steps -- in this case a leisurely 1 seconds"

	^ 1000! !

!ClipboardMorph methodsFor: 'testing' stamp: 'sw 6/27/2001 13:40'!
wantsSteps
	^ true! !

EllipseMorph subclass: #AtomMorph
	instanceVariableNames: 'velocity size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!AtomMorph commentStamp: 'tbn 11/25/2004 09:06' prior: 0!
AtomMorph represents an atom used in the simulation of
an ideal gas. It's container is typically a BouncingAtomsMorph.

Try:

	BouncingAtomsMorph  new openInWorld

to open the gas simulation or:

	AtomMorph example

to open an instance in the current world!

!AtomMorph class methodsFor: 'examples' stamp: 'tbn 11/25/2004 09:03'!
example
	"
	AtomMorph example
	"
	|a|
	a := AtomMorph new openInWorld. 
	a color: Color random.
 	[1000 timesRepeat:  [a bounceIn: World bounds.  (Delay forMilliseconds: 50) wait]. 
	 a delete] fork.! !

!AtomMorph class methodsFor: 'new-morph participation' stamp: 'di 6/22/97 09:07'!
includeInNewMorphMenu
	"Not to be instantiated from the menu"
	^ false! !

!AtomMorph methodsFor: 'private' stamp: 'CosteDucroz 2/15/2012 09:13'!
bounceIn: aRect
	"Move this atom one step along its velocity vector and make it bounce if it goes outside the given rectangle. Return true if it is bounced."

	| p vx vy px py bounced |

	p := self position.
	vx := velocity x.		vy := velocity y.
	px := p x + vx.		py := p y + vy.
	bounced := false.
	px > aRect right ifTrue: [
		px := aRect right - (px - aRect right).
		vx := velocity x negated.
		bounced := true].
	py > aRect bottom ifTrue: [
		py :=  aRect bottom - (py - aRect bottom).
		vy := velocity y negated.
		bounced := true].
	px < aRect left ifTrue: [
		px := aRect left - (px - aRect left).
		vx := velocity x negated.
		bounced := true].
	py < aRect top ifTrue: [
		py :=  aRect top - (py - aRect top).
		vy := velocity y negated.
		bounced := true].
	self position: px @ py.
	bounced ifTrue: [self velocity: vx @ vy.].
	^ bounced
! !

!AtomMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:13'!
defaultBorderWidth
"answer the default border width for the receiver"
	^ 0! !

!AtomMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:13'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color blue! !

!AtomMorph methodsFor: 'drawing' stamp: ''!
drawOn: aCanvas
	"Note: Set 'drawAsRect' to true to make the atoms draw faster. When testing the speed of other aspects of Morphic, such as its damage handling efficiency for large numbers of atoms, it is useful to make drawing faster."

	| drawAsRect |
	drawAsRect := false.  "rectangles are faster to draw"
	drawAsRect
		ifTrue: [aCanvas fillRectangle: self bounds color: color]
		ifFalse: [super drawOn: aCanvas].! !

!AtomMorph methodsFor: 'accessing' stamp: ''!
infected

	^ color = Color red! !

!AtomMorph methodsFor: 'accessing' stamp: ''!
infected: aBoolean

	aBoolean
		ifTrue: [self color: Color red]
		ifFalse: [self color: Color blue].! !

!AtomMorph methodsFor: 'initialization' stamp: 'CosteDucroz 3/5/2012 16:57'!
initialize
	"Make a new atom with a random position and velocity."
	super initialize.
""
	self extent: 8 @ 8.
	
	self
		randomPositionIn: (0 @ 0 corner: 300 @ 300)
		maxVelocity: 10! !

!AtomMorph methodsFor: 'initialization' stamp: 'RAA 12/15/2000 07:32'!
randomPositionIn: aRectangle maxVelocity: maxVelocity
	"Give this atom a random position and velocity."

	| origin extent |
	origin := aRectangle origin.
	extent := (aRectangle extent - self bounds extent) rounded.
	self position:
		(origin x + extent x atRandom) @
		(origin y + extent y atRandom).
	velocity :=
		(maxVelocity - (2 * maxVelocity) atRandom) @
		(maxVelocity - (2 * maxVelocity) atRandom).
! !

!AtomMorph methodsFor: 'accessing' stamp: ''!
velocity

	^ velocity! !

!AtomMorph methodsFor: 'accessing' stamp: ''!
velocity: newVelocity

	velocity := newVelocity.! !

AtomMorph subclass: #CrossMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

CrossMorph class
	instanceVariableNames: 'crossHeight'!

CrossMorph class
	instanceVariableNames: 'crossHeight'!

!CrossMorph methodsFor: 'as yet unclassified' stamp: 'CosteDucroz 3/5/2012 16:17'!
drawOn: aCanvas
"crossHeight est la hauteur de la barre horizontale horizontalBar
et crossWidth est la largeur de la barre verticale verticalBar"
| crossHeight crossWidth horizontalBar verticalBar |
crossHeight := self height / 4.0 .
crossWidth := self width / 4.0.
horizontalBar := self bounds insetBy: -2 @ crossHeight.
verticalBar := self bounds insetBy: crossWidth @ -1.2.
aCanvas fillRectangle: horizontalBar color: self color.
aCanvas fillRectangle: verticalBar color: self color.! !

EllipseMorph subclass: #Flasher
	instanceVariableNames: 'onColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!Flasher commentStamp: 'StephaneDucasse 8/19/2009 22:56' prior: 0!
A simple example - a circle that flashes.

	Flasher new openInWorld 

The "onColor" instance variable indicates the color to use when "on",  A darker color is used to represent "off".

The #step method, called every 500ms. by default, alternatively makes the flasher show its "on" and its "off" color.!

!Flasher class methodsFor: 'parts bin' stamp: 'sw 4/17/2002 11:37'!
descriptionForPartsBin
	"Answer a description of the receiver for use in a parts bin"

	^ self partName:	'Flasher'
		categories:		#('Demo')
		documentation:	'A circle that flashes'! !

!Flasher methodsFor: 'parts bin' stamp: 'sd 4/21/2002 09:36'!
initializeToStandAlone
	"Initialize the flasher."

	super initializeToStandAlone.
	self color: Color red.
	self onColor: Color red. 
	self borderWidth: 2.
	self extent: 25@25! !

!Flasher methodsFor: 'operations' stamp: 'sw 5/28/2002 18:44'!
onColor
	"Answer my onColor"

	^ onColor ifNil: [onColor := Color red]! !

!Flasher methodsFor: 'operations' stamp: 'sd 4/21/2002 09:55'!
onColor: aColor
	"Change my on color to be aColor"

	onColor := aColor.
	self color: aColor! !

!Flasher methodsFor: 'stepping and presenter' stamp: 'sw 5/28/2002 18:45'!
step
	"Perform my standard periodic action"

	super step.
	self color = self onColor
		ifTrue: [self color: (onColor alphaMixed: 0.5 with: Color black)]
		ifFalse: [self color: onColor]! !

!Flasher methodsFor: 'testing' stamp: 'sw 4/17/2002 12:05'!
stepTime
	"Answer the desired time between steps, in milliseconds."

	^ 500! !

EllipseMorph subclass: #MovingEyeMorph
	instanceVariableNames: 'inner iris'
	classVariableNames: 'IrisSize'
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!MovingEyeMorph commentStamp: 'StephaneDucasse 8/20/2009 09:40' prior: 0!
A MovingEyeMorph is a morph representing an eye whose iris is following the mouse cursor.!

!MovingEyeMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 12:51'!
descriptionForPartsBin
	^ self partName:	'MovingEye'
		categories:		#('Demo')
		documentation:	'An eye which follows the cursor'! !

!MovingEyeMorph class methodsFor: 'initialization' stamp: 'yo 2/15/2001 16:04'!
initialize
"
	MovingEyeMorph initialize
"
	IrisSize := (0.42@0.50).! !

!MovingEyeMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:42'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color black! !

!MovingEyeMorph methodsFor: 'geometry' stamp: 'yo 2/15/2001 15:59'!
extent: aPoint

	super extent: aPoint.
	inner extent: (self extent * ((1.0@1.0)-IrisSize)) asIntegerPoint.
	iris extent: (self extent * IrisSize) asIntegerPoint.
	inner position: (self center - (inner extent // 2)) asIntegerPoint.
! !

!MovingEyeMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:42'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	inner := EllipseMorph new.
	inner color: self color.
	inner extent: (self extent * (1.0 @ 1.0 - IrisSize)) asIntegerPoint.
	inner borderColor: self color.
	inner borderWidth: 0.
""
	iris := EllipseMorph new.
	iris color: Color white.
	iris extent: (self extent * IrisSize) asIntegerPoint.
""
	self addMorphCentered: inner.
	inner addMorphCentered: iris.
""
	self extent: 26 @ 33! !

!MovingEyeMorph methodsFor: 'as yet unclassified' stamp: 'yo 2/15/2001 15:24'!
irisPos: cp

	| a b theta x y |
	theta := (cp - self center) theta.
	a := inner width // 2.
	b := inner height // 2.
	x := a * (theta cos).
	y := b * (theta sin).
	iris position: ((x@y) asIntegerPoint) + self center - (iris extent // 2).! !

!MovingEyeMorph methodsFor: 'stepping and presenter' stamp: 'di 2/18/2001 00:10'!
step
	| cp |
	cp := self globalPointToLocal: World primaryHand position.
	(inner containsPoint: cp)
		ifTrue: [iris position: (cp - (iris extent // 2))]
		ifFalse: [self irisPos: cp].
	self changed "cover up gribblies if embedded in Flash"! !

!MovingEyeMorph methodsFor: 'testing' stamp: 'yo 2/15/2001 15:38'!
stepTime

	^ 100.! !

EllipseMorph subclass: #WatchMorph
	instanceVariableNames: 'fontName cColor handsColor romanNumerals antialias'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!WatchMorph commentStamp: '<historical>' prior: 0!
This class is a representation of a watch.
The labels' font is changeble. Labels' font size increase or decrease when resizing me.

WatchMorph new openInWorld
(WatchMorph fontName: 'ComicPlain' bgColor: Color transparent centerColor: Color transparent) openInWorld		" transparent "
(WatchMorph fontName: 'ComicBold' bgColor: Color white centerColor: Color black) openInWorld

Structure:
	fontName		String -- the labels' font name
	cColor			Color -- center color
	handsColor		Color
	romanNumerals	Boolean
	antialias		Boolean!

!WatchMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 12:53'!
descriptionForPartsBin
	^ self partName:	'Watch'
		categories:		#('Useful')
		documentation:	'An analog clock face'! !

!WatchMorph class methodsFor: 'as yet unclassified' stamp: 'di 5/16/2000 21:43'!
fontName: aString bgColor: aColor centerColor: otherColor
	^ self new
		fontName: aString;
		color: aColor;
		centerColor: otherColor! !

!WatchMorph methodsFor: 'menus' stamp: 'mara 7/17/2010 16:36'!
addCustomMenuItems: aMenu hand: aHandMorph
	"Add morph-specific items to the given menu which was invoked by the given hand."

	super addCustomMenuItems: aMenu hand: aHandMorph.
	aMenu addLine.
	aMenu addUpdating: #romanNumeralString action: #toggleRoman.
	aMenu addUpdating: #antiAliasString action: #toggleAntialias.
	aMenu addLine.
	aMenu add: 'change font...' translated action: #changeFont.
	aMenu balloonTextForLastItem: 'Allows you to change the font used to display the numbers.' translated.
	aMenu balloonTextForLastItem: 'Allows you to specify a new color for the hands of the watch.  Note that actual *watch* color can be changed simply by using the halo''s recoloring handle.' translated.
	aMenu balloonTextForLastItem: 'Allows you to specify a new color to be used during PM hours for the center portion of the watch; during AM hours, a lighter shade of the same color will be used.' translated.! !

!WatchMorph methodsFor: 'menus' stamp: 'dgd 8/30/2003 22:25'!
antiAliasString
	^ (antialias
		ifTrue: ['<on>']
		ifFalse: ['<off>'])
		, 'anti-aliasing' translated! !

!WatchMorph methodsFor: 'accessing' stamp: 'bf 5/18/2000 11:06'!
antialias: aBoolean
	antialias := aBoolean! !

!WatchMorph methodsFor: 'accessing' stamp: 'sw 7/4/2002 00:19'!
centerColor: aColor
	"Set the center color as indicated; map nil into transparent"

	cColor := aColor ifNil: [Color transparent]! !

!WatchMorph methodsFor: 'menus' stamp: 'alain.plantec 2/6/2009 17:47'!
changeFont

	| choice |
	choice := UIManager default chooseFrom: StrikeFont familyNames values: StrikeFont familyNames.
	choice ifNotNil: [self fontName: choice]
! !

!WatchMorph methodsFor: 'labels' stamp: 'di 5/16/2000 21:42'!
createLabels

	| numeral font h r |
	self removeAllMorphs.
	font := StrikeFont familyName: fontName size: (h := self height min: self width)//8.
	r := 1.0 - (1.4 * font height / h).
	1 to: 12 do:
		[:hour |
		numeral := romanNumerals
			ifTrue: [#('I' 'II' 'III' 'IV' 'V' 'VI' 'VII' ' VIII' 'IX' 'X' 'XI' 'XII') at: hour]
			ifFalse: [hour asString].
		self addMorphBack: ((StringMorph contents: numeral font: font emphasis: 1)
			center: (self radius: r hourAngle: hour)) lock].
! !

!WatchMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:01'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color green! !

!WatchMorph methodsFor: 'drawing' stamp: 'sw 7/4/2002 00:22'!
drawOn: aCanvas
	"Draw the watch on the given canvas"

	| pHour pMin pSec time centerColor |
	time := Time now.
	pHour := self radius: 0.6 hourAngle: time hours + (time minutes/60.0).
	pMin := self radius: 0.72 hourAngle: (time minutes / 5.0).
	pSec := self radius: 0.8 hourAngle: (time seconds / 5.0).
	centerColor := cColor
		ifNil:
			[Color transparent]
		ifNotNil:
			[time hours < 12
				ifTrue: [cColor muchLighter]
				ifFalse: [cColor]].

	antialias ifTrue:
		[aCanvas asBalloonCanvas
			aaLevel: 4;
			drawOval: (bounds insetBy: borderWidth // 2 + 1) color: self fillStyle
				borderWidth: borderWidth borderColor: borderColor;
			drawOval: (bounds insetBy: self extent*0.35) color: centerColor
				borderWidth: 0 borderColor: Color black;
			drawPolygon: {self center. pHour}
				color: Color transparent borderWidth: 3 borderColor: handsColor;
			drawPolygon: {self center. pMin}
				color: Color transparent borderWidth: 2 borderColor: handsColor;
			drawPolygon: {self center. pSec}
				color: Color transparent borderWidth: 1 borderColor: handsColor]
		ifFalse:
			[super drawOn: aCanvas.
			aCanvas
				fillOval: (bounds insetBy: self extent*0.35) color: centerColor;
				line: self center to: pHour width: 3 color: handsColor;
				line: self center to: pMin width: 2 color: handsColor;
				line: self center to: pSec width: 1 color: handsColor]
! !

!WatchMorph methodsFor: 'geometry' stamp: 'di 5/16/2000 21:18'!
extent: newExtent

	super extent: newExtent.
	self createLabels! !

!WatchMorph methodsFor: 'accessing' stamp: 'di 5/16/2000 21:45'!
fontName: aString

	fontName := aString.
	self createLabels! !

!WatchMorph methodsFor: 'accessing' stamp: 'bf 9/25/2008 15:17'!
handsColor: aColor

	handsColor := aColor! !

!WatchMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 15:01'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""

	self handsColor: Color red.
	self centerColor: Color gray.
	romanNumerals := false.
	antialias := false.
	fontName := 'NewYork'.
	self extent: 130 @ 130.
	self start! !

!WatchMorph methodsFor: 'private' stamp: 'di 5/16/2000 22:22'!
radius: unitRadius hourAngle: hourAngle
	"unitRadius goes from 0.0 at the center to 1.0 on the circumference.
	hourAngle runs from 0.0 clockwise around to 12.0 with wrapping."

	^ self center + (self extent * (Point r: 0.5 * unitRadius
									degrees: hourAngle * 30.0 - 90.0)).! !

!WatchMorph methodsFor: 'menus' stamp: 'dgd 8/30/2003 22:24'!
romanNumeralString
	"Answer a string governing the roman-numerals checkbox"
	^ (romanNumerals
		ifTrue: ['<on>']
		ifFalse: ['<off>'])
		, 'roman numerals' translated! !

!WatchMorph methodsFor: 'stepping and presenter' stamp: 'bf 9/25/2008 15:17'!
step

	self changed.! !

!WatchMorph methodsFor: 'menus' stamp: 'bf 5/17/2000 18:46'!
toggleAntialias
	antialias := antialias not! !

!WatchMorph methodsFor: 'menus' stamp: 'di 5/16/2000 21:19'!
toggleRoman

	romanNumerals := romanNumerals not.
	self createLabels! !

RectangleMorph subclass: #DoubleClickExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!DoubleClickExample commentStamp: 'StephaneDucasse 8/19/2009 22:55' prior: 0!
Illustrates the double-click capabilities of Morphic.

DoubleClickExample new openInWorld 	

If you have a kind of morph you wish to have respond specially to a double-click, it should:

(1)  Respond "true" to #handlesMouseDown:

(2)  In its mouseDown: method, send #waitForClicksOrDrag:event: to the hand.

(3)  Reimplement #click: to react to single-clicked mouse-down.

(4)  Reimplement #doubleClick: to make the appropriate response to a double-click.

(5)  Reimplement #drag: to react to non-clicks.  This message is sent continuously until the button is released.  You can check the event argument to react differently on the first, intermediate, and last calls.!

!DoubleClickExample class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 12:46'!
descriptionForPartsBin
	^ self partName:	'DoubleClick'
		categories:		#('Demo')
		documentation:	'An example of how to use double-click in moprhic'! !

!DoubleClickExample methodsFor: 'accessing' stamp: 'nk 7/26/2004 10:38'!
balloonText
	^ 'Double-click on me to change my color; 
single-click on me to change border color;
hold mouse down within me and then move it to grow 
(if I''m red) or shrink (if I''m blue).' translated
! !

!DoubleClickExample methodsFor: 'event handling' stamp: 'ar 10/3/2000 17:05'!
click: evt
	self showBalloon: 'click' hand: evt hand.
	self borderColor: (self borderColor = Color black ifTrue: [Color yellow] ifFalse: [Color black])
! !

!DoubleClickExample methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:22'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color red! !

!DoubleClickExample methodsFor: 'event handling' stamp: 'ar 10/3/2000 17:05'!
doubleClick: evt
	self showBalloon: 'doubleClick' hand: evt hand.
	self color: ((color = Color blue) ifTrue: [Color red] ifFalse: [Color blue])
! !

!DoubleClickExample methodsFor: 'event handling' stamp: 'sw 9/14/1999 16:05'!
handlesMouseDown: evt
	^ true! !

!DoubleClickExample methodsFor: 'event handling' stamp: 'bf 9/28/1999 17:20'!
mouseDown: evt
	"Do nothing upon mouse-down except inform the hand to watch for a double-click; wait until an ensuing click:, doubleClick:, or drag: message gets dispatched"

	evt hand waitForClicksOrDrag: self event: evt! !

!DoubleClickExample methodsFor: 'event handling' stamp: 'ar 10/3/2000 17:05'!
startDrag: evt
	"We'll get a mouseDown first, some mouseMoves, and a mouseUp event last"
	| oldCenter |
	evt isMouseDown ifTrue:
		[self showBalloon: 'drag (mouse down)' hand: evt hand.
		self world displayWorld.
		(Delay forMilliseconds: 750) wait].
	evt isMouseUp ifTrue:
		[self showBalloon: 'drag (mouse up)' hand: evt hand].
	(evt isMouseUp or: [evt isMouseDown]) ifFalse:
		[self showBalloon: 'drag (mouse still down)' hand: evt hand].
	(self containsPoint: evt cursorPoint)
		ifFalse: [^ self].

	oldCenter := self center.
	color = Color red
		ifTrue:
			[self extent: self extent + (1@1)]
		ifFalse:
			[self extent: ((self extent - (1@1)) max: (16@16))].
	self center: oldCenter! !

RectangleMorph subclass: #HeadingMorph
	instanceVariableNames: 'degrees magnitude'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!HeadingMorph commentStamp: 'StephaneDucasse 8/19/2009 23:00' prior: 0!
A HeadingMorph displays an arrow on a grid as well as its decomposition on x and y axis.
Clicking on it changes the direction of the main arrow.

	HeadingMorph new openInWorld!

!HeadingMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:34'!
defaultBorderWidth
"answer the default border width for the receiver"
	^ 1! !

!HeadingMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:33'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color
		r: 0.6
		g: 1.0
		b: 1.0! !

!HeadingMorph methodsFor: 'accessing' stamp: ''!
degrees

	^ (degrees + 90.0) \\ 360.0! !

!HeadingMorph methodsFor: 'accessing' stamp: ''!
degrees: aNumber

	degrees := (aNumber asFloat + 270.0) \\ 360.0.! !

!HeadingMorph methodsFor: 'drawing' stamp: ''!
drawArrowFrom: p1 to: p2 width: w color: aColor on: aCanvas

	| d p |
	d := (p1 - p2) theta radiansToDegrees.
	aCanvas line: p1 to: p2 width: w color: aColor.
	p := p2 + (Point r: 5 degrees: d - 50).
	aCanvas line: p to: p2 width: w color: aColor.
	p := p2 + (Point r: 5 degrees: d + 50).
	aCanvas line: p to: p2 width: w color: aColor.
! !

!HeadingMorph methodsFor: 'drawing' stamp: ''!
drawOn: aCanvas

	| x y r center box |
	super drawOn: aCanvas.
	box := self innerBounds.
	1 to: 9 do: [:i |
		x := box left + ((box width * i) // 10).
		aCanvas line: (x@box top) to: (x@(box bottom - 1)) color: 
Color black.
		y := box top + ((box height * i) // 10).
		aCanvas line: (box left@y) to: ((box right - 1)@y) color: 
Color black].

	r := ((box width asFloat * magnitude asFloat) / 2.0) - 1.0.
	center := box center.
	self drawArrowFrom: center - (1@1)
		to: center + ((r * degrees degreesToRadians cos)@0) - (1@1)
		width: 3
		color: (Color red)
		on: aCanvas.
	self drawArrowFrom: center - (1@1)
		to: center + (0@(r * degrees degreesToRadians sin)) - (1@1)
		width: 3
		color: (Color red)
		on: aCanvas.
	self drawArrowFrom: center - (1@1)
		to: center + (Point r: r degrees: degrees) - (1@1)
		width: 3
		color: Color black
		on: aCanvas.
! !

!HeadingMorph methodsFor: 'geometry' stamp: ''!
extent: aPoint
	"Contrain extent to be square."

	| d |
	d := aPoint x min: aPoint y.
	super extent: d@d.
! !

!HeadingMorph methodsFor: 'event handling' stamp: ''!
handlesMouseDown: evt

	^ true
! !

!HeadingMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:34'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	degrees := 90.0.
	magnitude := 1.0.
	
	self extent: 160 @ 160! !

!HeadingMorph methodsFor: 'accessing' stamp: ''!
magnitude

	^ magnitude! !

!HeadingMorph methodsFor: 'accessing' stamp: ''!
magnitude: aNumber

	magnitude := (aNumber asFloat max: 0.0) min: 1.0.! !

!HeadingMorph methodsFor: 'events' stamp: 'mk 11/7/2003 11:35'!
mouseDown: evt

	| v |
	self changed.
	v := evt cursorPoint - bounds center.
	degrees := v theta radiansToDegrees.
	magnitude := (v r asFloat / (bounds width asFloat / 2.0)) min: 1.0.
! !

!HeadingMorph methodsFor: 'events' stamp: 'mk 11/7/2003 11:36'!
mouseMove: evt

	self mouseDown: evt! !

RectangleMorph subclass: #StarWithABallMorph
	instanceVariableNames: 'phase ball star'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!StarWithABallMorph commentStamp: 'StephaneDucasse 8/20/2009 09:40' prior: 0!
This is a example of how to use a morph. It consists of only two 
methods, initialize and step.

DoIt:
StarWithABall new openInWorld.



!

!StarWithABallMorph methodsFor: 'initialization' stamp: 'dgd 2/21/2003 19:59'!
initialize
	"initialize the state of the receiver"
	super initialize.
	phase := 1.
	self extent: 200 @ 200.
	ball := EllipseMorph new extent: 30 @ 30.
	self
		addMorph: ((star := StarMorph new extent: 150 @ 150) center: self center)! !

!StarWithABallMorph methodsFor: 'stepping and presenter' stamp: 'kfr 10/26/2003 18:33'!
step
	phase := phase\\8 + 1.
	phase = 1 ifTrue: [^ ball delete].
	phase < 4 ifTrue:[^self].
	phase = 4 ifTrue: [self addMorph: ball].
	ball align: ball center with: (star vertices at: (phase-3*2)).! !

RectangleMorph subclass: #StickyPadMorph
	instanceVariableNames: ''
	classVariableNames: 'Colors LastColorIndex'
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!StickyPadMorph commentStamp: 'StephaneDucasse 8/20/2009 10:16' prior: 0!
A custom item for the  Squeakland Supplies bin, as defined by Kim Rose and BJ Con.A parts bin will deliver up translucent, borderless Rectangles in a sequence of 6 colors.  It offers some complication to the parts-bin protocols in two ways::
* The multi-colored icon seen in the parts bin is not a thumbnail of any actual instance, all of which are monochrome
* New instances need to be given default names that are not the same as the name seen in the parts bin.

StickyPadMorph minimics the stickyPad 
	StickyPadMorph new openInWorld 

	!

!StickyPadMorph class methodsFor: 'parts bin' stamp: 'sw 4/4/2003 11:12'!
descriptionForPartsBin
	"Answer a description of the receiver for use in a parts bin"

	^ self partName: 	'Sticky Pad'
		categories:		#('Graphics')
		documentation:	'A translucent, borderless rectangle of a standard size, delivered in a predictable sequence of pastel colors'
		sampleImageForm: (Form extent: 50@40 depth: 16
	fromArray: #( 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461414680 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1796762392 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461414680 1796762392 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1796762392 1796762392 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461409563 1461414680 1796762392 1796762392 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1723098804 1723098804 1723098804 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521903284 1723098804 1723098804 1723096921 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1723098804 1723098804 1723098804 1599692633 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521903284 1723098804 1723098804 1723096921 1599692633 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1723098804 1723098804 1723098804 1599692633 1599692633 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1521900214 1521900214 1521900214 1521903284 1723098804 1723098804 1723096921 1599692633 1599692633 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1322274512 1322274512 1322274512 1389318863 1389318863 1389318863 1328697138 1328697138 1328697138 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1322274512 1322274512 1322275535 1389318863 1389318863 1389317938 1328697138 1328697138 1328702226 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1322274512 1322274512 1389318863 1389318863 1389318863 1328697138 1328697138 1328697138 1662149394 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1322274512 1322275535 1389318863 1389318863 1389317938 1328697138 1328697138 1328702226 1662149394 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1322274512 1389318863 1389318863 1389318863 1328697138 1328697138 1328697138 1662149394 1662149394 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1322275535 1389318863 1389318863 1389317938 1328697138 1328697138 1328702226 1662149394 1662149394 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521900214 1389318863 1389318863 1389318863 1460426508 1460426508 1460426508 1659658988 1659658988 1659658988 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1521903284 1389318863 1389318863 1389317938 1460426508 1460426508 1460429548 1659658988 1659658988 1659660157 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521900214 1723098804 1389318863 1389318863 1328697138 1460426508 1460426508 1659658988 1659658988 1659658988 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1521903284 1723098804 1389318863 1389317938 1328697138 1460426508 1460429548 1659658988 1659658988 1659660157 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521900214 1723098804 1723098804 1389318863 1328697138 1328697138 1460426508 1659658988 1659658988 1659658988 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1521903284 1723098804 1723098804 1389317938 1328697138 1328697138 1460429548 1659658988 1659658988 1659660157 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461409563 1723098804 1723098804 1723098804 1328697138 1328697138 1328697138 1659658988 1659658988 1659658988 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1461414680 1723098804 1723098804 1723096921 1328697138 1328697138 1328702226 1659658988 1659658988 1659660157 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461409563 1796762392 1723098804 1723098804 1599692633 1328697138 1328697138 1662149394 1659658988 1659658988 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1461414680 1796762392 1723098804 1723096921 1599692633 1328697138 1328702226 1662149394 1659658988 1659660157 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461409563 1796762392 1796762392 1723098804 1599692633 1599692633 1328697138 1662149394 1662149394 1659658988 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1736271741 1461414680 1796762392 1796762392 1723096921 1599692633 1599692633 1328702226 1662149394 1662149394 1659660157 1736271741 1736271741 1736271741 1736271741 1736271741)
	offset: 0@0)! !

!StickyPadMorph class methodsFor: 'initialization' stamp: 'sw 3/3/2004 13:44'!
initialize
	"Class initialization"

	LastColorIndex := 0.
	Colors :=  {
		TranslucentColor r: 0.0 g: 0.0 b: 0.839 alpha: 0.267.
		TranslucentColor r: 0.484 g: 1.0 b: 0.452 alpha: 0.706.
		TranslucentColor r: 1.0 g: 0.355 b: 0.71 alpha: 0.569.
		TranslucentColor r: 1.0 g: 1.0 b: 0.03 alpha: 0.561.
		TranslucentColor r: 0.484 g: 0.161 b: 1.0 alpha: 0.529.
		TranslucentColor r: 0.097 g: 0.097 b: 0.097 alpha: 0.192.
	}.
	
	self registerInFlapsRegistry.	

"StickyPadMorph initialize"! !

!StickyPadMorph class methodsFor: 'as yet unclassified' stamp: 'sw 3/3/2004 13:42'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(StickyPadMorph		newStandAlone			'Sticky Pad'			'Each time you obtain one of these pastel, translucent, borderless rectangles, it will be a different color from the previous time.')
						forFlapNamed: 'Supplies'.
				cl registerQuad: #(StickyPadMorph		newStandAlone			'Sticky Pad'			'Each time you obtain one of these pastel, translucent, borderless rectangles, it will be a different color from the previous time.')
						forFlapNamed: 'PlugIn Supplies'.]! !

!StickyPadMorph methodsFor: 'visual properties' stamp: 'dgd 9/18/2004 18:52'!
canHaveFillStyles
	"Return true if the receiver can have general fill styles; not just 
	colors. This method is for gradually converting old morphs."
	^ true! !

!StickyPadMorph methodsFor: 'parts bin' stamp: 'StephaneDucasse 8/20/2009 09:59'!
initialize
	"Initialize the receiver to stand alone.  Use the next color in the standard sequence."

	Colors ifNil: [self class initialize].
	LastColorIndex := 
		LastColorIndex
			ifNil:
				[1]
			ifNotNil:
				[(LastColorIndex \\ Colors size) + 1].
	super initialize.
	self assureExternalName.
	self color: (Colors at: LastColorIndex).
	self extent: 200@160.
	self borderWidth: 0
	! !

PolygonMorph subclass: #StarMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!StarMorph commentStamp: 'wiz 9/6/2005 12:30' prior: 0!
I am a very flexible star..

Grab me from the supplies flap or from the graphic objects.
Add my handles and you can move and resize me.
The up and down arrows increase or reduce the number of my sides.
The right and left arrows cycle thru different amounts of  pointiness.


Use the arrows right and left of my center or get stars with a specific amount of pointyness.  The left side goes from fat to thin and then cycles around again. The right goes from thin to fat. Hold down the shift key if you wish to stop the cycling at the extremes.

Use the arrows up and down to change the number of sides if you would like a different number of points.

To add or remove just one side hold the shift key down as you use the arrows or use the menu items for that purpose.


If you add or remove just one point I will have an odd number of sides.  When that happens I can only look like a regular polygon. The right and left arrows will have no effect.  Add or remove just one more side and you can shift drag the outer handle or use the arrows to restore my pointiness. 

That was too complicated. It is gone. You can get regular polygon shapes by adjusting my pointiness. For example the extreme of a five pointed star is a dodecahedron (10 sided regular polygon) and one step less extreme is a pentagon (5 sided regular polygon).


At some time you will probably shift drag the outer handle thru the center handle.
While I looked round as you shrunk me, I will look very much like an asterisk as you pull me away.  What happens is that inside bend shrunk on the way down because it can never be larger than the outer point (or it wouldn't be the innerbend would it).
But on the way out it is perfectly happy to remain small. So I look like an asterisk.

To fatten me up  (if you haven't already figured this out by fooling around)  hold the shift down an move the outer handle towards the center (but not quite all the way) then let the shift up and move the outer handle away.  A couple of cycles like this and I'll be looking fat and jolly again. Or you can now just use the right arrow to make me fatter.

This is also the reason I don't let the inside bend get larger than the outer point.
If I did the same process that fattened me when I was an asterisk would also grow an asterisk so large squeak would complain about not having enough memory.

Historical note:

The former star had two bugs that are fixed here.
The outer handle now no longer jumps from one point to another.
The other bug prevented some higher order stars from looking right. 
Which is why the former star didn't allow you to change the number of points. !

!StarMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 16:22'!
descriptionForPartsBin
	^ self partName:	'Star'
		categories:		#('Graphics')
		documentation:	'A symmetrical polygon in the shape of a star'! !

!StarMorph class methodsFor: 'initialization' stamp: 'asm 4/11/2003 11:37'!
initialize

	self registerInFlapsRegistry.	! !

!StarMorph class methodsFor: 'initialization' stamp: 'asm 4/11/2003 11:39'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(StarMorph		authoringPrototype	'Star'	'A star')
						forFlapNamed: 'PlugIn Supplies'.
						cl registerQuad: #(StarMorph	authoringPrototype	'Star'	'A star')
						forFlapNamed: 'Supplies'.]! !

!StarMorph class methodsFor: 'initialization' stamp: 'asm 4/11/2003 12:41'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!StarMorph methodsFor: 'menu' stamp: 'wiz 9/5/2005 23:08'!
addChangeSidesMenuItems: aCustomMenu hand: aHandMorph 
	"Menu items to change number of sides."
	aCustomMenu addLine.
	aCustomMenu add: 'more sides' translated action: #moreVertices.
	aCustomMenu add: 'fewer sides' translated action: #lessVertices.
"	Regular polygons can be simulated with the one of the skip stars 
	and it would confuse users to have stars be limited to Regular polygons.
	So we've removed those menu items - wiz"
"	aCustomMenu add: 'one more side' translated action: #oneMoreVertex.
	aCustomMenu add: 'one fewer side' translated action: #oneLessVertex"! !

!StarMorph methodsFor: 'menu' stamp: 'wiz 1/11/2005 14:55'!
addCustomMenuItems: aCustomMenu hand: aHandMorph 
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
self addChangeSidesMenuItems: aCustomMenu hand: aHandMorph.
self addTwinkleMenuItems: aCustomMenu hand: aHandMorph.
	! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 12/15/2004 00:59'!
addHandles
	self addStarHandles! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 1/15/2005 00:20'!
addStarHandles
	"Outer handle must not be blocked so it comes first. 
	The editing routine expects to find the center handle second.
	The side and shape changing handles follow these."
	| center |
	self removeHandles.
	"Check for old stars and correct order of vertices."
	self insureCompatability .
	handles := OrderedCollection new.
	center := vertices average rounded.
	self withCenterOuterHandles; withUpDownLeftRightHandlesAround: 6 center: center.
	self placeHandles.
	self changed.
	! !

!StarMorph methodsFor: 'menu' stamp: 'wiz 1/11/2005 20:38'!
addTwinkleMenuItems: aCustomMenu hand: aHandMorph 
	"Menu items to change the sharpness of the star."
	aCustomMenu addLine.
	aCustomMenu add: 'twinkle fatter' translated action: #nextTwinkle.
	aCustomMenu add: 'twinkle thinner' translated action: #prevTwinkle.
"	aCustomMenu add: 'fatter star' translated action: #nextFatter.
	aCustomMenu add: 'thinner star' translated action: #prevThinner"
	
	


! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 9/5/2005 23:12'!
changeVertices: label event: evt fromHandle: handle 
	| |
	label == #more
		ifTrue: [evt shiftPressed
				ifTrue: [self moreVertices "not oneMoreVertex"]
				ifFalse: [self moreVertices]].
	label == #less
		ifTrue: [evt shiftPressed
				ifTrue: [self lessVertices "not oneLessVertex"]
				ifFalse: [self lessVertices]].
	label == #next
		ifTrue: [evt shiftPressed
				ifTrue: [self makeVertices: vertices size starRatio: self nextSkip]
				ifFalse: [self makeVertices: vertices size starRatio: self nextTwinkleSkip]].
	label == #prev
		ifTrue: [evt shiftPressed
				ifTrue: [self makeVertices: vertices size starRatio: self prevSkip]
				ifFalse: [self makeVertices: vertices size starRatio: self prevTwinkleSkip]].
	self computeBounds! !

!StarMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:35'!
defaultBorderColor
	"answer the default border color/fill style for the receiver"
	^ Color black! !

!StarMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:40'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 1! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/11/2005 14:33'!
defaultCenter
	"answer the default center for the receiver"
	^ 0 asPoint! !

!StarMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:31'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color lightBlue! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/11/2005 14:33'!
defaultFirstVertex
	"answer the default first outer point for the receiver.
	This with the center determines the angle and size of the outer radius."
	^ 10 asPoint! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/11/2005 14:28'!
defaultSides
	"answer the default number of sides for the receiver"
	^ 10! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/11/2005 14:29'!
defaultStarRatio
	"answer the default ratio of outer radius to inner radius for the receiver"
	^ 5.0 / 12.0! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 1/20/2005 19:07'!
dragVertex: label event: evt fromHandle: handle 
	| center r1 rN rNext a1 rTotal |
	label == #outside
		ifTrue: [center := handles second center.
			r1 := center dist: vertices first.
			"Rounding and what happens as the outer handle
			approached the center, 
			requires we guard the inner radius 
			from becoming larger than the outer radius."
			rN := r1
						min: (center dist: vertices last).
			rNext := 1
						max: (center dist: evt cursorPoint).
			a1 := 270.0
						+ (center bearingToPoint: evt cursorPoint).
			rTotal := vertices size even
						ifTrue: [evt shiftPressed
								ifTrue: [rNext + rNext min: rNext + rN]
								ifFalse: [r1 + rN * rNext / r1]]
						ifFalse: [rNext + rNext].
			rNext := rTotal - rNext.
			vertices := ((a1 to: a1 + 359.999 by: 360.0 / vertices size)
						collect: [:angle | center
								+ (Point r: (rNext := rTotal - rNext) degrees: angle)]) .
			handle align: handle center with: evt cursorPoint].
	label == #center
		ifTrue: [evt shiftPressed
				ifTrue: [self updateFormFromUser]
				ifFalse: [self position: self position + (evt cursorPoint - handle center)]].
	self computeBounds! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/11/2005 14:36'!
initialize
	"initialize the state of the receiver"
	super initialize.
	self
		makeVertices: self defaultSides
		starRatio: self defaultStarRatio
		withCenter: self defaultCenter
		withPoint: self defaultFirstVertex.
	self computeBounds! !

!StarMorph methodsFor: 'parts bin' stamp: 'sw 7/2/2001 11:07'!
initializeToStandAlone
	^ self initialize removeHandles! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/9/2005 20:15'!
insureCompatability
"The old stars had the point on the second not the first vertex. So we need to check for this special case."
 | c v1 v2 |
c := vertices average rounded.
 v1 := vertices first .
 v2 := vertices second .
(c dist: v1) + 0.001 < (c dist: v2) ifTrue: [vertices := vertices allButFirst copyWith: v1]

! !

!StarMorph methodsFor: 'menus' stamp: 'wiz 12/12/2004 16:43'!
lessVertices
"Reduce the number of points by one until we are  a diamond. If odd reduce the number of sides by two until we become a triangle. See class comment."
	| nVerts |
	( nVerts := 2 negated + vertices size) < 3 ifFalse: [
	self
		makeVertices: nVerts]! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 12/30/2004 02:57'!
makeVertices: nSides 
	"Assuming vertices has at least one point, make a new star 
	or regular polygon (for odd sided polygons).
	The center of the polygon and the first vertex remain in
	place. The inner distances for stars remain the same also if
	possible."
	| center r1 rN rNext a1 rTotal |
	center := vertices average rounded.
	r1 := center dist: vertices first.
	rN := center dist: vertices last.
	rNext := 1 max: r1.
	a1 := 270.0
				+ (center bearingToPoint: vertices first).
	rTotal := nSides even
				ifTrue: [rNext + rNext min: rNext + rN]
				ifFalse: [rNext + rNext].
	rNext := rTotal - rNext.
	self changed .
	vertices := (a1 to: a1 + 359.999 by: 360.0 / nSides)
				collect: [:angle | center
						+ (Point r: (rNext := rTotal - rNext) degrees: angle)].
	self computeBounds.
	self changed! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/4/2005 19:31'!
makeVertices: nSides starRatio: fraction
	"Assuming vertices has at least one point, make a new star 
	or regular polygon (for odd sided polygons).
	The center of the polygon and the first vertex remain in
	place. The inner distances for stars remain the same also if
	possible."
	| center r1 rN rNext a1 rTotal |
	center := vertices average rounded.
	r1 := center dist: vertices first.
	rNext := 1 max: r1.
	rN := (1.0 min: fraction) * rNext.
	a1 := 270.0
				+ (center bearingToPoint: vertices first).
	rTotal := nSides even
				ifTrue: [rNext + rNext min: rNext + rN]
				ifFalse: [rNext + rNext].
	rNext := rTotal - rNext.
	self changed .
	vertices := (a1 to: a1 + 359.999 by: 360.0 / nSides)
				collect: [:angle | center
						+ (Point r: (rNext := rTotal - rNext) degrees: angle)].
	self computeBounds.
	self changed! !

!StarMorph methodsFor: 'initialization' stamp: 'wiz 1/11/2005 21:36'!
makeVertices: nSides starRatio: fraction withCenter: center withPoint: aPoint 
	"Make a new star or regular polygon (for odd sided polygons).
	This makes star vertices from scratch without any feedback from existing vertices."
	| r1 rN rNext a1 rTotal |
	r1 := center dist: aPoint.
	rNext := 1 max: r1.
	rN := (1.0 min: fraction)
				* rNext.
	a1 := 270.0
				+ (center bearingToPoint: aPoint).
	rTotal := nSides even
				ifTrue: [rNext + rNext min: rNext + rN]
				ifFalse: [rNext + rNext].
	rNext := rTotal - rNext.
	self changed.
	vertices := (a1 to: a1 + 359.999 by: 360.0 / nSides)
				collect: [:angle | center
						+ (Point r: (rNext := rTotal - rNext) degrees: angle)].
	self computeBounds.
	self changed! !

!StarMorph methodsFor: 'menus' stamp: 'wiz 12/11/2004 16:45'!
moreVertices
	self makeVertices: 2+ vertices size! !

!StarMorph methodsFor: 'geometry' stamp: 'wiz 1/19/2005 22:34'!
nextSkip
	"Set starRatio to next skip wrapping if needed."
	| skips n c r1 rN |
	c := vertices average rounded.
	r1 := (c dist: vertices first) truncated asFloat.
	rN := c dist: vertices last.
	skips := self skipRatios.
	n := skips * r1
				findFirst: [:r | r > (rN + 1.0)].
	"n = 0
		ifTrue: [n := skips size]."
	^ skips atWrap: n! !

!StarMorph methodsFor: 'menus' stamp: 'wiz 1/11/2005 20:37'!
nextTwinkle
	self makeVertices: vertices size starRatio: self nextTwinkleSkip .
	self computeBounds.! !

!StarMorph methodsFor: 'geometry' stamp: 'wiz 1/19/2005 22:42'!
nextTwinkleSkip
	"Set starRatio to next skip wrapping if needed."
	| skips n c r1 rN |
	c := vertices average rounded.
	r1 := (c dist: vertices first) truncated asFloat.
	rN := c dist: vertices last.
	skips := self skipRatios.
	n := skips * r1
				findFirst: [:r | r > (rN + 1.0)].
	n = 0
		ifTrue: [ n := 1].
	^ skips atWrap: n! !

!StarMorph methodsFor: 'menu' stamp: 'wiz 12/11/2004 16:45'!
oneLessVertex
	self
		makeVertices: (3 max: 1 negated + vertices size)! !

!StarMorph methodsFor: 'menu' stamp: 'wiz 12/11/2004 16:46'!
oneMoreVertex
	self makeVertices: 1 + vertices size! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 1/15/2005 00:19'!
placeHandles
	"Add the handles to my submorphs."
	handles reverseDo: [:each | self addMorphFront: each ] .
	
	! !

!StarMorph methodsFor: 'geometry' stamp: 'wiz 1/19/2005 22:41'!
prevSkip
	"Set starRatio to next skip wrapping if necessary"
	| skips n c r1 rN |
	c := vertices average rounded.
	r1 := c dist: vertices first.
	rN := (c dist: vertices last) truncated asFloat.
	skips := self skipRatios.
	n := skips * r1
				findLast: [:r | r + 1.0 < rN].
	n = 0
		ifTrue: [n := 1].
	^ skips at: n! !

!StarMorph methodsFor: 'menus' stamp: 'wiz 1/11/2005 20:37'!
prevTwinkle
	self makeVertices: vertices size starRatio: self prevTwinkleSkip .
	self computeBounds.! !

!StarMorph methodsFor: 'geometry' stamp: 'wiz 1/19/2005 22:19'!
prevTwinkleSkip
	"Set starRatio to next skip wrapping if necessary"
	| skips n c r1 rN |
	c := vertices average rounded.
	r1 := c dist: vertices first.
	rN := (c dist: vertices last) truncated asFloat.
	skips := self skipRatios.
	n := skips * r1
				findLast: [:r | r + 1.0 < rN].
	"n = 0
	ifTrue: [^ oldR]."
	^ skips atWrap: n! !

!StarMorph methodsFor: 'access' stamp: 'wiz 1/11/2005 03:58'!
skipRatios
"Return an array of  ratios  of the inner radius to the outer radius.
Ratios are in ascending order from 0.0 to 1.0."
"Assume we have at least one vertex.
All ways return a number <= 1.0"

| n  alpha  |
"Odd vertices sizes can not be stars only regular polygons"
n:= vertices size . n odd ifTrue: [ ^ #(  1.0) ] .

alpha := Float pi / (n//2)  asFloat .

^ ((((    Float halfPi -alpha  to: alpha /2.0  by: alpha  negated ) 
	collect:  [:angle |( (angle) sin )/
					(angle + alpha ) sin ]
	) copyWith: 0.0) copyWithFirst: 1.0) reversed .! !

!StarMorph methodsFor: 'access' stamp: 'wiz 1/4/2005 20:14'!
starRatio
"Return the ratio of the inner radius to the outer radius."
"Assume we have at least one vertex.
All ways return a number <= 1.0"
| r c |
c := vertices average rounded .
r := (c dist: vertices last) / (c dist:  vertices first)  .
^ r > 1.0  ifTrue: [  r reciprocal ] ifFalse: [r ] .! !

!StarMorph methodsFor: 'accessing' stamp: 'wiz 1/4/2005 19:47'!
starRatio: r
"Set the star s.t. the ratio of the inner radius to the outer radius is r.
If r is > 1 use the reciprocal to keep the outer radius first."
"Assume we have at least one vertex.
set
All ways return a number <= 1.0"
self makeVertices: vertices size starRatio:( r > 1.0  ifTrue: [  r reciprocal ] ifFalse: [r ] ).! !

!StarMorph methodsFor: 'menus' stamp: 'wiz 1/15/2005 02:26'!
updateFormFromUser
"Does nothing here. Overridden in subclasses e.g. Kaleidoscope."
^ self.! !

!StarMorph methodsFor: 'editing' stamp: 'di 9/26/97 11:11'!
updateHandles! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 1/11/2005 19:39'!
withCenterOuterHandles
	"Add to our handles the center positioning and outer resizing
	handles. Outer handle must not be blocked so it comes first. 
	The editing routine expects to find the center handle second.
	The side and shape changing handles follow these."
	| center v1 hExtent holder |
	center := vertices average rounded.
	hExtent := 8 @ 8.
	v1 := vertices first.
	holder := {(EllipseMorph
				newBounds: (Rectangle center: v1 extent: hExtent)
				color: Color yellow)
				setBalloonText: 'Move me to adjust size. Shift move to adjust pointiness'. (EllipseMorph
				newBounds: (Rectangle center: center extent: hExtent)
				color: Color yellow)
				setBalloonText: 'Move me to adjust position'}.
	holder
		with: {#outside. #center}
		do: [:handle :which | handle
				on: #mouseDown
				send: #dragVertex:event:fromHandle:
				to: self
				withValue: which;
				
				on: #mouseMove
				send: #dragVertex:event:fromHandle:
				to: self
				withValue: which].
	handles addAll: holder! !

!StarMorph methodsFor: 'editing' stamp: 'wiz 1/11/2005 19:47'!
withUpDownLeftRightHandlesAround: radius center: center
	"Add to our handles the side and shape changing handles."
	| tri  above holder  triAbove triBelow triRight triLeft |
	above := 0 @ radius negated.
	
	tri := Array
				with: 0 @ -5
				with: 4 @ 3
				with: -4 @ 3.
	triAbove := tri + (center + above).
	triBelow := triAbove
				collect: [:pt | pt rotateBy: #pi centerAt: center].
	triRight := triAbove
				collect: [:pt | pt rotateBy: #right centerAt: center].
	triLeft := triAbove
				collect: [:pt | pt rotateBy: #left centerAt: center].
				
	holder := { (PolygonMorph
				vertices: triAbove
				color: Color green
				borderWidth: 1
				borderColor: Color black)
				 setBalloonText: 'More points.'. 
				
				(PolygonMorph
				vertices: triBelow
				color: Color magenta
				borderWidth: 1
				borderColor: Color black)
				 setBalloonText: 'Fewer points.'. 
				
				(PolygonMorph
				vertices: triRight
				color: Color green
				borderWidth: 1
				borderColor: Color black)
				 setBalloonText: 'Twinkle fatter.'. 
				
				(PolygonMorph
				vertices: triLeft
				color: Color magenta
				borderWidth: 1
				borderColor: Color black)
				 setBalloonText: 'Twinkle thinner.'}.
	
	holder
		with: {#more. #less. #next. #prev}
		do: [:handle :which | handle
				on: #mouseDown
				send: #changeVertices:event:fromHandle:
				to: self
				withValue: which;
				
				on: #mouseMove
				send: #changeVertices:event:fromHandle:
				to: self
				withValue: which].
	^ handles addAll: holder! !

StringMorph subclass: #ClockMorph
	instanceVariableNames: 'showSeconds show24hr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!ClockMorph commentStamp: 'StephaneDucasse 8/19/2009 22:54' prior: 0!
A ClockMorph is oneline clock

	ClockMorph new openInWorld 
	ClockMorph new openInHand!

!ClockMorph class methodsFor: 'scripting' stamp: 'sw 10/16/1998 15:36'!
authoringPrototype
	^ super authoringPrototype contents: Time now printString! !

!ClockMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 01:22'!
descriptionForPartsBin
	^ self partName:	'Clock'
		categories:		#('Useful')
		documentation:	'A digital clock'! !

!ClockMorph class methodsFor: 'initialization' stamp: 'asm 4/10/2003 13:00'!
initialize

	self registerInFlapsRegistry.	! !

!ClockMorph class methodsFor: 'initialization' stamp: 'asm 4/10/2003 13:02'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(ClockMorph	authoringPrototype		'Clock'			'A simple digital clock')
						forFlapNamed: 'Supplies'.
						cl registerQuad: #(ClockMorph	authoringPrototype		'Clock'			'A simple digital clock')
						forFlapNamed: 'PlugIn Supplies'.]! !

!ClockMorph class methodsFor: 'initialization' stamp: 'asm 4/11/2003 12:33'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!ClockMorph methodsFor: 'menu' stamp: 'stephane.ducasse 2/14/2009 17:36'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	"Note minor loose end here -- if the menu is persistent, then the wording will be wrong half the time"
	| item |
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	item := showSeconds 
		ifTrue:	['stop showing seconds']
		ifFalse: ['start showing seconds'].
	aCustomMenu add: item translated target: self action: #toggleShowingSeconds.
	item := show24hr
		ifTrue: ['display Am/Pm']
		ifFalse: ['display 24 hour'].
	aCustomMenu add: item translated target: self action: #toggleShowing24hr.	
		
! !

!ClockMorph methodsFor: 'initialization' stamp: 'stephane.ducasse 2/14/2009 17:37'!
initialize
	"initialize the state of the receiver"
	
	super initialize.
	showSeconds := true.
	show24hr := false.
	self step! !

!ClockMorph methodsFor: 'parts bin' stamp: 'sw 7/12/2001 17:41'!
initializeToStandAlone
	super initializeToStandAlone.
	showSeconds := true.
	self step! !

!ClockMorph methodsFor: '24hr' stamp: 'fc 2/8/2004 11:38'!
show24hr: aBoolean
	show24hr := aBoolean! !

!ClockMorph methodsFor: 'seconds' stamp: 'sw 2/17/1999 14:39'!
showSeconds: aBoolean
	showSeconds := aBoolean! !

!ClockMorph methodsFor: 'stepping and presenter' stamp: 'stephane.ducasse 2/14/2009 17:37'!
step

	| time |
	super step.
	time := String streamContents:
		[:aStrm | Time now print24: show24hr  showSeconds: showSeconds on: aStrm].
	self contents: time			! !

!ClockMorph methodsFor: 'testing' stamp: ''!
stepTime
	"Answer the desired time between steps in milliseconds."

	^ 1000! !

!ClockMorph methodsFor: '24hr' stamp: 'stephane.ducasse 2/14/2009 17:37'!
toggleShowing24hr
	show24hr := show24hr not
! !

!ClockMorph methodsFor: 'seconds' stamp: 'stephane.ducasse 2/14/2009 17:37'!
toggleShowingSeconds
	showSeconds := showSeconds not
! !

StringMorph subclass: #FrameRateMorph
	instanceVariableNames: 'lastDisplayTime framesSinceLastDisplay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!FrameRateMorph commentStamp: 'StephaneDucasse 8/19/2009 22:59' prior: 0!
A little Morph to show the speed of frame rate of the system.

	FrameRateMorph new openInWorld.
	FrameRateMorph new openInHand.

!

!FrameRateMorph class methodsFor: 'scripting' stamp: 'sw 6/13/2001 00:57'!
authoringPrototype
	"Answer a morph representing a prototypical instance of the receiver"

	| aMorph |
	aMorph := self new.
	aMorph color: Color blue.
	aMorph step.
	^ aMorph! !

!FrameRateMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 12:49'!
descriptionForPartsBin
	^ self partName:	'FrameRate'
		categories:		#('Useful')
		documentation:	'A readout that allows you to monitor the frame rate of your system'! !

!FrameRateMorph class methodsFor: 'initialization' stamp: 'asm 4/10/2003 13:05'!
initialize

	self registerInFlapsRegistry.	! !

!FrameRateMorph class methodsFor: 'initialization' stamp: 'asm 4/10/2003 13:06'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(FrameRateMorph		authoringPrototype		'Frame Rate'		'An indicator of how fast your system is running')
						forFlapNamed: 'Widgets']! !

!FrameRateMorph class methodsFor: 'initialization' stamp: 'asm 4/11/2003 12:36'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!FrameRateMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 20:42'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	lastDisplayTime := 0.
	framesSinceLastDisplay := 0! !

!FrameRateMorph methodsFor: 'parts bin' stamp: 'sw 7/19/2001 13:39'!
initializeToStandAlone
	"Initialize the receiver as a stand-alone entity"

	super initializeToStandAlone.
	self color: Color blue.
	self step! !

!FrameRateMorph methodsFor: 'stepping and presenter' stamp: 'sw 10/5/2000 06:52'!
step
	"Compute and display (every half second or so) the current framerate"

	| now mSecs mSecsPerFrame framesPerSec newContents |
	framesSinceLastDisplay := framesSinceLastDisplay + 1.
	now := Time millisecondClockValue.
	mSecs := now - lastDisplayTime.
	(mSecs > 500 or: [mSecs < 0 "clock wrap-around"]) ifTrue: 
		[mSecsPerFrame := mSecs // framesSinceLastDisplay.
		framesPerSec := (framesSinceLastDisplay * 1000) // mSecs.
		newContents := mSecsPerFrame printString, ' mSecs (', framesPerSec printString, ' frame', (framesPerSec == 1 ifTrue: [''] ifFalse: ['s']), '/sec)'.
		self contents: newContents.
		lastDisplayTime := now.
		framesSinceLastDisplay := 0]! !

!FrameRateMorph methodsFor: 'testing' stamp: 'jm 2/23/98 18:41'!
stepTime
	"Answer the desired time between steps in milliseconds."

	^ 0
! !

StringMorph subclass: #InfoStringMorph
	instanceVariableNames: 'stepTime block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!InfoStringMorph commentStamp: 'StephaneDucasse 8/19/2009 23:37' prior: 0!
A generalization of the clock morph

Try
	InfoStringMorph new openInWorld
or
	(InfoStringMorph on: [SmalltalkImage current  vmParameterAt: 9])
		stepTime: 50;
		openInWorld!

!InfoStringMorph class methodsFor: 'instance creation' stamp: 'sma 6/1/2000 14:07'!
on: aBlock
	^ self new block: aBlock! !

!InfoStringMorph methodsFor: 'accessing' stamp: 'sma 6/1/2000 14:06'!
block
	^ block! !

!InfoStringMorph methodsFor: 'accessing' stamp: 'sma 6/1/2000 14:05'!
block: aBlock
	block := aBlock! !

!InfoStringMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 20:42'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	stepTime := 1000.
	block := [Time now]! !

!InfoStringMorph methodsFor: 'stepping and presenter' stamp: 'sma 6/1/2000 14:03'!
step
	self contents: block value asString! !

!InfoStringMorph methodsFor: 'testing' stamp: 'sma 6/1/2000 14:03'!
stepTime
	^ stepTime! !

!InfoStringMorph methodsFor: 'accessing' stamp: 'sma 6/1/2000 14:06'!
stepTime: anInteger
	stepTime := anInteger! !

BorderedMorph subclass: #MagnifierMorph
	instanceVariableNames: 'magnification trackPointer srcExtent showPointer'
	classVariableNames: 'RecursionLock'
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!MagnifierMorph commentStamp: 'TestRunner 11/14/2009 16:57' prior: 0!
MagnifierMorph instances are magnifying lenses that magnify the morphs below them (if grabbed or if trackPointer is false) or the area around the mouse pointer.

	MagnifierMorph newShowingPointer openInWorld 	
	MagnifierMorph new openInWorld
	

Instance variables:

magnification	<Number> The magnification to use. If non-integer, smooths the magnified form.

trackPointer		<Boolean> If set, magnifies the area around the Hand. If not, magnfies the area underneath the magnifier center.

showPointer		<Boolean> If set, display a small reversed rectangle in the center of the lens. Also enables the display of Morphs in the Hand itself.

srcExtent		<Rectangle> The extent of the source rectangle.
		
Class variables:

RecursionLock	<MagnifierMorph|nil> Used to avoid infinite recursion when getting the source patch to display.!

MagnifierMorph subclass: #FishEyeMorph
	instanceVariableNames: 'gridNum d clipRects toRects quads savedExtent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!FishEyeMorph commentStamp: 'StephaneDucasse 8/19/2009 23:43' prior: 0!
A FishEyeMorph is an extreme-wide-angle lens MagnifierMorph

	FishEyeMorph new openInWorld!

!FishEyeMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 12:47'!
descriptionForPartsBin
	^ self partName:	'FishEye'
		categories:		#('Useful')
		documentation:	'An extreme-wide-angle lens'! !

!FishEyeMorph methodsFor: 'initialization' stamp: 'yo 12/17/1999 12:00'!
calculateTransform
	| stepX stepY rect tx ty arrayX arrayY |
	(gridNum x = 0 or: [gridNum y = 0]) ifTrue: [^self].
	stepX := srcExtent x // gridNum x.
	stepY := srcExtent y // gridNum y.

	arrayX := (1 to: gridNum y + 1) collect: [:j | FloatArray new: gridNum x + 1].
	arrayY := (1 to: gridNum y + 1) collect: [:j |  FloatArray new: gridNum x + 1].

	0 to: gridNum y do: [:j |
		0 to: gridNum x do: [:i |
			(arrayX at: (j + 1)) at: (i + 1) put: i*stepX.
			(arrayY at: (j + 1)) at: (i + 1) put: j*stepY.
		].
	].

	0 to: gridNum y do: [:j |
		self transformX: (arrayX at: (j+1)).
		self transformY: (arrayY at: (j+1)).
	].

	0 to: gridNum y do: [:j |
		arrayX at: (j+1) put: ((1 to: gridNum x +1) collect: [:i | ((arrayX at: (j+1)) at: i) asInteger]).
		arrayY at: (j+1) put: ((1 to: gridNum x +1) collect: [:i | ((arrayY at: (j+1)) at: i) asInteger]).
	].


	clipRects := (1 to: gridNum y) collect: [:j | Array new: gridNum x].
	toRects := (1 to: gridNum y) collect: [:j |  Array new: gridNum x].
	quads := (1 to: gridNum y) collect: [:j |  Array new: gridNum x].
	0 to: gridNum y - 1 do: [:j |
		0 to: gridNum x- 1 do: [:i |
			rect := (((arrayX at: (j+1)) at: (i+1))@((arrayY at: (j+1)) at: (i+1)))
						corner: ((arrayX at: (j+2)) at: (i+2))@((arrayY at: (j+2)) at: (i+2)).
			(clipRects at: j+1) at: i+1 put: rect.

			rect width >= stepX ifTrue: [rect := rect expandBy: (1@0)].
			rect height >= stepY ifTrue: [rect := rect expandBy: (0@1)].
			(toRects at: j+1) at: i+1 put: rect.

			tx := (i)*stepX.
			ty := (j)*stepY.
			(quads at: j+1) at: i+1
						put: {(tx)@(ty). (tx)@(ty+stepY). (tx+stepX)@(ty+stepY). (tx+stepX)@(ty)}.
		].
	].

! !

!FishEyeMorph methodsFor: 'menus' stamp: 'dgd 9/21/2003 17:55'!
chooseMagnification
	self inform: 'Magnification is fixed, sorry.' translated! !

!FishEyeMorph methodsFor: 'menu' stamp: 'yo 12/17/1999 12:03'!
chooseMagnification: evt
! !

!FishEyeMorph methodsFor: 'geometry' stamp: 'yo 12/17/1999 12:27'!
extent: aPoint
	"Round to a number divisible by grid.  Note that the superclass has its own implementation."
	| g gridSize |
	gridSize := self gridSizeFor: aPoint.
	"self halt."
	g := (aPoint - (2 * borderWidth)) // gridSize.
	srcExtent := g * gridSize.
	gridNum := g.
	^super extent: self defaultExtent! !

!FishEyeMorph methodsFor: 'initialization' stamp: 'yo 12/17/1999 10:15'!
g: aFloatArray max: max focus: focus
	| dNormX array |

	dNormX := aFloatArray - focus.
	
	array := dNormX / max.
	array *= d.
	array += 1.0.
	array := 1.0 / array.
	dNormX *= (d+1.0).
	array *= dNormX.
	^array += focus.
! !

!FishEyeMorph methodsFor: 'private' stamp: 'yo 12/17/1999 11:15'!
gridSizeFor: aPoint
	"returns appropriate size for specified argument"
	| g |
	g := aPoint x min: aPoint y.
	g <= 256 ifTrue: [^8].
	^16.! !

!FishEyeMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 21:39'!
initialize
"initialize the state of the receiver"
	super initialize.
""
	"magnification should be always 1"
	magnification := 1.
	d := 1.3.
	self extent: 130 @ 130! !

!FishEyeMorph methodsFor: 'parts bin' stamp: 'sw 6/28/2001 11:32'!
initializeToStandAlone
	super initializeToStandAlone.
	"magnification should be always 1"
	magnification := 1.
	d := 1.3.
	self extent: 130@130.
! !

!FishEyeMorph methodsFor: 'magnifying' stamp: 'ar 5/28/2000 12:12'!
magnifiedForm
	| warp warpForm fromForm |

	savedExtent ~= srcExtent ifTrue: [
		savedExtent := srcExtent.
		self calculateTransform].

	warpForm := Form extent: srcExtent depth: Display depth.
	fromForm := super magnifiedForm.

	warp :=  (WarpBlt current toForm: warpForm)
		sourceForm: fromForm;
		colorMap: nil;
		cellSize: 2;
		combinationRule: Form over.

	1 to: gridNum y do: [:j |
		1 to: gridNum x do: [:i |
			warp
				clipRect: ((clipRects at: j) at: i);
				copyQuad: ((quads at: j) at: i)
					toRect: ((toRects at: j) at: i).
		].
	].
	^warpForm
! !

!FishEyeMorph methodsFor: 'initialization' stamp: 'md 11/14/2003 16:32'!
transformX: aFloatArray
	| focus gridNum2 subArray dMaxX |

	focus := srcExtent x asFloat / 2.

	gridNum2 := (aFloatArray findFirst: [:x | x > focus]) - 1.

	dMaxX := 0.0 - focus.
	subArray := self g: (aFloatArray copyFrom: 1 to: gridNum2) max: dMaxX focus: focus.

	aFloatArray replaceFrom: 1 to: gridNum2 with: subArray startingAt: 1.


	dMaxX := focus.    " = (size - focus)"
	subArray := self g: (aFloatArray copyFrom: gridNum2 + 1 to: gridNum x + 1)
		max: dMaxX focus: focus.

	aFloatArray replaceFrom: gridNum2 + 1 to: gridNum x + 1 with: subArray startingAt: 1.
! !

!FishEyeMorph methodsFor: 'initialization' stamp: 'dgd 2/21/2003 23:04'!
transformY: aFloatArray 
	| focus subArray dMaxY |
	focus := srcExtent y asFloat / 2.
	dMaxY := (aFloatArray first) <= focus 
				ifTrue: [0.0 - focus]
				ifFalse: [focus].
	subArray := self 
				g: (aFloatArray copyFrom: 1 to: gridNum x + 1)
				max: dMaxY
				focus: focus.
	aFloatArray 
		replaceFrom: 1
		to: gridNum x + 1
		with: subArray
		startingAt: 1! !

!MagnifierMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 15:04'!
descriptionForPartsBin
	^ self partName:	'Magnifier'
		categories:		#('Useful')
		documentation:	'A magnifying glass'! !

!MagnifierMorph class methodsFor: 'instance creation' stamp: 'nk 3/6/2004 10:28'!
newShowingPointer
	"Answer a Magnifier that also displays Morphs in the Hand and the Hand position"

	^(self new)
		showPointer: true;
		setNameTo: 'HandMagnifier';
		yourself! !

!MagnifierMorph methodsFor: 'menu' stamp: 'TestRunner 11/14/2009 17:01'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu
		addLine;
		add: 'magnification...' translated action: #chooseMagnification;
		addUpdating: #trackingPointerString action: #toggleTrackingPointer;
		addUpdating: #showingPointerString action: #toggleShowingPointer! !

!MagnifierMorph methodsFor: 'accessing' stamp: 'bf 9/21/1999 09:31'!
borderWidth: anInteger
	"Grow outwards preserving innerBounds"
	| c |  
	c := self center.
	super borderWidth: anInteger.
	super extent: self defaultExtent.
	self center: c.! !

!MagnifierMorph methodsFor: 'menu' stamp: 'alain.plantec 2/6/2009 17:08'!
chooseMagnification
	| result possibleMagnifications title |
	possibleMagnifications := #(1.5 2 4 8).
	title := 'Choose magnification' translated, '\(' , 'currently' translated, ' {1})' format: {magnification}.
	result := UIManager default chooseFrom: possibleMagnifications values: possibleMagnifications title: title.
	(result isNil or: [result = magnification]) ifTrue: [^ self].
	magnification := result.
	self extent: self extent. "round to new magnification"
	self changed. "redraw even if extent wasn't changed"! !

!MagnifierMorph methodsFor: 'menu' stamp: 'di 8/24/2000 14:02'!
chooseMagnification: evt
	| handle origin aHand currentMag |
	currentMag := magnification.
	aHand := evt ifNil: [self currentHand] ifNotNil: [evt hand].
	origin := aHand position y.
	handle := HandleMorph new forEachPointDo:
		[:newPoint | self magnification: (newPoint y - origin) / 8.0 + currentMag].
	aHand attachMorph: handle.
	handle startStepping.
	self changed. "Magnify handle"! !

!MagnifierMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:38'!
defaultBorderWidth
	"answer the default border width for the receiver"
	^ 1! !

!MagnifierMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:28'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color black! !

!MagnifierMorph methodsFor: 'geometry' stamp: 'bf 9/21/1999 09:22'!
defaultExtent
	^(srcExtent * magnification) truncated + (2 * borderWidth)! !

!MagnifierMorph methodsFor: 'drawing' stamp: 'ar 2/12/2000 18:06'!
drawOn: aCanvas
	super drawOn: aCanvas.		"border and fill"
	aCanvas isShadowDrawing ifFalse: [
		"Optimize because #magnifiedForm is expensive"
		aCanvas paintImage: self magnifiedForm at: self innerBounds origin]! !

!MagnifierMorph methodsFor: 'geometry' stamp: 'bf 9/21/1999 09:23'!
extent: aPoint
	"Round to multiples of magnification"
	srcExtent := (aPoint - (2 * borderWidth)) // magnification.
	^super extent: self defaultExtent! !

!MagnifierMorph methodsFor: 'event handling' stamp: 'bf 9/18/1999 20:42'!
handlesMouseDown: evt
	^evt yellowButtonPressed
		or: [super handlesMouseDown: evt]! !

!MagnifierMorph methodsFor: 'accessing' stamp: 'bf 9/21/1999 08:51'!
hasTranslucentColor
	"I may show what's behind me, so tell the hand to don't cache"
	^self sourceRect intersects: self bounds! !

!MagnifierMorph methodsFor: 'initialization' stamp: 'nk 3/6/2004 10:47'!
initialize
	"initialize the state of the receiver"
	super initialize.

	trackPointer := true.
	showPointer := false.
	magnification := 2.

	self extent: 128 @ 128! !

!MagnifierMorph methodsFor: 'magnifying' stamp: 'bf 9/21/1999 12:27'!
magnification: aNumber
	| c |  
	magnification := aNumber min: 8 max: 0.5.
	magnification := magnification roundTo:
		(magnification < 3 ifTrue: [0.5] ifFalse: [1]).
	srcExtent := srcExtent min: (512@512) // magnification. "to prevent accidents"
	c := self center.
	super extent: self defaultExtent.
	self center: c.! !

!MagnifierMorph methodsFor: 'magnifying' stamp: 'TestRunner 11/14/2009 17:02'!
magnifiedForm
	"Answer the magnified form"
	| srcRect form magnified |
	srcRect := self sourceRectFrom: self sourcePoint.
	(RecursionLock isNil and: [ self showPointer or: [ srcRect intersects: self bounds ]])
		ifTrue: [RecursionLock := self.
			form := self currentWorld
						patchAt: srcRect
						without: self
						andNothingAbove: false.
			RecursionLock := nil]
		ifFalse: ["cheaper method if the source is not occluded"
			form := Display copy: srcRect].
	"smooth if non-integer scale"
	magnified := form
				magnify: form boundingBox
				by: magnification
				smoothing: (magnification isInteger
						ifTrue: [1]
						ifFalse: [2]).
	"display the pointer rectangle if desired"
	self showPointer
		ifTrue: [magnified
				reverse: (magnified center - (2 @ 2) extent: 4 @ 4)
				fillColor: Color white].
	^ magnified! !

!MagnifierMorph methodsFor: 'event handling' stamp: 'bf 9/21/1999 10:45'!
mouseDown: evt
	evt yellowButtonPressed
		ifTrue: [self chooseMagnification: evt]
		ifFalse: [super mouseDown: evt]! !

!MagnifierMorph methodsFor: 'menu' stamp: 'nk 3/17/2004 11:34'!
showPointer
	^showPointer ifNil: [ showPointer := false ].! !

!MagnifierMorph methodsFor: 'accessing' stamp: 'nk 3/6/2004 10:14'!
showPointer: aBoolean
	"If aBoolean is true, display the current pointer position as a small square in the center of the lens."

	showPointer == aBoolean ifTrue: [ ^self ].
	showPointer := aBoolean.
	self changed.! !

!MagnifierMorph methodsFor: 'menu' stamp: 'nk 3/17/2004 11:34'!
showingPointerString
	^ (self showPointer
		ifTrue: ['stop showing pointer']
		ifFalse: ['start showing pointer']) translated! !

!MagnifierMorph methodsFor: 'magnifying' stamp: 'bf 11/1/2000 16:02'!
sourcePoint
	"If we are being dragged use our center, otherwise use pointer position"
	^(trackPointer not or: [owner notNil and: [owner isHandMorph]])
		ifTrue: [self center]
		ifFalse: [self currentHand position]! !

!MagnifierMorph methodsFor: 'magnifying' stamp: 'bf 9/21/1999 08:47'!
sourceRect
	^self sourceRectFrom: self sourcePoint
! !

!MagnifierMorph methodsFor: 'magnifying' stamp: 'di 12/17/1999 13:42'!
sourceRectFrom: aPoint
	^ (aPoint extent: srcExtent) translateBy: (srcExtent // -2) + 1.
! !

!MagnifierMorph methodsFor: 'stepping and presenter' stamp: 'di 9/28/1999 07:57'!
step
	self changed! !

!MagnifierMorph methodsFor: 'testing' stamp: 'di 9/28/1999 07:57'!
stepTime
	^ 0! !

!MagnifierMorph methodsFor: 'menu' stamp: 'nk 3/17/2004 11:35'!
toggleShowingPointer
	self showPointer: self showPointer not! !

!MagnifierMorph methodsFor: 'menu' stamp: 'bf 9/20/1999 15:48'!
toggleTrackingPointer
	trackPointer := trackPointer not! !

!MagnifierMorph methodsFor: 'menu' stamp: 'dgd 8/30/2003 21:51'!
trackingPointerString
	^ (trackPointer
		ifTrue: ['stop tracking pointer']
		ifFalse: ['start tracking pointer']) translated! !

ImageMorph subclass: #AnimatedImageMorph
	instanceVariableNames: 'images delays stepTime nextTime imageIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!AnimatedImageMorph commentStamp: 'StephaneDucasse 8/20/2009 10:40' prior: 0!
I am an ImageMorph that can hold more than one image. Each image has its own delay time.

If you have an animated gif close to the image named 

	(AnimatedImageMorph fromGIFFileNamed: 'animated.gif') openInWorld !

!AnimatedImageMorph class methodsFor: 'filein/out' stamp: 'nk 6/12/2004 13:11'!
fileReaderServicesForFile: fullName suffix: suffix

	^((AnimatedGIFReadWriter typicalFileExtensions asSet
		add: '*'; add: 'form'; yourself)
		includes: suffix)
		ifTrue: [ self services ]
		ifFalse: [#()]
! !

!AnimatedImageMorph class methodsFor: 'instance creation' stamp: 'nk 2/15/2004 15:23'!
fromGIFFileNamed: fileName
	| reader |
	reader := AnimatedGIFReadWriter formsFromFileNamed: fileName.
	^reader forms size = 1
		ifTrue: [ ImageMorph new image: reader forms first ]
		ifFalse: [ self new fromReader: reader ]! !

!AnimatedImageMorph class methodsFor: 'instance creation' stamp: 'nk 2/15/2004 15:27'!
fromStream: aStream
	| reader |
	reader := AnimatedGIFReadWriter formsFromStream: aStream.
	^reader forms size = 1
		ifTrue: [ ImageMorph new image: reader forms first ]
		ifFalse: [ self new fromReader: reader ]! !

!AnimatedImageMorph class methodsFor: 'initialization' stamp: 'asm 12/11/2003 21:05'!
initialize
	"register the receiver in the global registries"
	self environment
		at: #FileList
		ifPresent: [:cl | cl registerFileReader: self]! !

!AnimatedImageMorph class methodsFor: 'instance creation' stamp: 'nk 2/15/2004 16:57'!
openGIFInWindow: aStream
	^(self fromStream: aStream binary) openInWorld! !

!AnimatedImageMorph class methodsFor: 'filein/out' stamp: 'dgd 4/3/2006 13:36'!
serviceOpenGIFInWindow
	"Answer a service for opening a gif graphic in a window"
	^ (SimpleServiceEntry
		provider: self
		label: 'open the graphic as a morph'
		selector: #openGIFInWindow:
		description: 'open a GIF graphic file as a morph'
		buttonLabel: 'open')
		argumentGetter: [:fileList | fileList readOnlyStream]! !

!AnimatedImageMorph class methodsFor: 'filein/out' stamp: 'dgd 4/3/2006 13:36'!
services
	^ Array
		with: self serviceOpenGIFInWindow
		"with: Form serviceImageImports"
		with: Form serviceImageAsBackground! !

!AnimatedImageMorph class methodsFor: 'initialization' stamp: 'asm 12/11/2003 21:01'!
unload
	"Unload the receiver from global registries"
	self environment
		at: #FileList
		ifPresent: [:cl | cl unregisterFileReader: self]! !

!AnimatedImageMorph methodsFor: 'private' stamp: 'nk 2/15/2004 15:20'!
fromGIFFileNamed: fileName
	self fromReader: (AnimatedGIFReadWriter formsFromFileNamed: fileName)! !

!AnimatedImageMorph methodsFor: 'private' stamp: 'bf 2/25/2005 11:18'!
fromReader: reader
	images := reader forms.
	delays := reader delays.
	imageIndex := 0.
	self image: (Form extent: images first extent depth: 32).
	self step! !

!AnimatedImageMorph methodsFor: 'private' stamp: 'nk 2/15/2004 15:20'!
fromStream: aStream
	self fromReader: (AnimatedGIFReadWriter formsFromStream: aStream)! !

!AnimatedImageMorph methodsFor: 'private' stamp: 'mir 11/19/2003 13:42'!
images
	^images! !

!AnimatedImageMorph methodsFor: 'private' stamp: 'bf 2/25/2005 11:09'!
initialize
	nextTime := Time millisecondClockValue.
	imageIndex := 1.
	stepTime := 10.
	super initialize! !

!AnimatedImageMorph methodsFor: 'stepping and presenter' stamp: 'marcus.denker 9/14/2008 19:13'!
step
	| d |
	images isEmpty ifTrue: [^ self].
		
	nextTime > Time millisecondClockValue ifTrue: [^self].
	self changed.
	self image: (images at: (imageIndex := imageIndex \\ images size + 1)).
	self changed . 
	d := (delays at: imageIndex) ifNil: [0].
	nextTime := Time millisecondClockValue + d
! !

!AnimatedImageMorph methodsFor: 'stepping and presenter' stamp: 'mir 11/19/2003 13:40'!
stepTime
	^stepTime ifNil: [super stepTime]! !

!AnimatedImageMorph methodsFor: 'stepping and presenter' stamp: 'mir 11/19/2003 13:40'!
stepTime: anInteger
	stepTime := anInteger! !

!AnimatedImageMorph methodsFor: 'stepping and presenter' stamp: 'asm 12/15/2003 19:44'!
wantsSteps
	^(images size > 1)
! !

ImageMorph subclass: #ImageMorphWithSpotlight
	instanceVariableNames: 'spotImage spotShape spotBuffer spotOn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!ImageMorphWithSpotlight commentStamp: 'StephaneDucasse 8/19/2009 23:34' prior: 0!
This class implements an image that appears one way or another depending upon whether it lies within a spotlight shape that moves with the cursor.  As delivered, the shape is a simple circle, the spotlighted appearance is that of a ColorForm, and the non-highlighted apperarance is a derived gray-scale form.

	(ImageMorphWithSpotlight new image: (Form fromUser)) openInWorld 
	

The implementation will space-efficient if supplied with a ColorForm, because the gray-scale derived form will share the same bitmap.
In general, any two images can be used -- one could be blurred, the other sharp -- and the masking form can be any shape.
At some point this class should be merged somehow with ScreeningMorph.

-- Note from Stephane Ducasse - August 2009: Does not seem to work with ColorForm

!

!ImageMorphWithSpotlight class methodsFor: 'example' stamp: 'StephaneDucasse 8/19/2009 23:05'!
example1
	"self example1"
	
	(self new image: (Form fromUser)) openInWorld! !

!ImageMorphWithSpotlight methodsFor: 'all' stamp: 'di 5/26/1999 07:51'!
backgroundImage: bkgndImage spotImage: anImage spotShape: formOfDepth1

	"See class comment."
	spotImage := anImage.
	spotShape := formOfDepth1.
	spotBuffer := Form extent: spotShape extent depth: spotImage depth.
	super image: bkgndImage.
	spotOn := false.! !

!ImageMorphWithSpotlight methodsFor: 'drawing' stamp: 'ar 2/12/2000 18:06'!
drawOn: aCanvas

	super drawOn: aCanvas.
	spotOn ifTrue:
		[aCanvas paintImage: spotBuffer at: spotBuffer offset].
! !

!ImageMorphWithSpotlight methodsFor: 'accessing' stamp: 'di 5/26/1999 08:05'!
image: anImage

	"The spotlight will reveal the original  form supplied
	while the background form will be derived grayscale."
	"See class comment."
	self backgroundImage: anImage asGrayScale
		spotImage: anImage
		spotShape: (Form dotOfSize: 100)
! !

!ImageMorphWithSpotlight methodsFor: 'all' stamp: 'di 5/24/1999 22:37'!
spotChanged

	self invalidRect:
		((spotBuffer offset extent: spotBuffer extent) "intersect: self bounds")! !

!ImageMorphWithSpotlight methodsFor: 'stepping and presenter' stamp: 'ar 5/28/2000 12:09'!
step
	| cp |
	((self bounds expandBy: spotBuffer extent // 2) containsPoint: (cp := self cursorPoint))
		ifTrue:
		[(cp - (spotBuffer extent // 2)) = spotBuffer offset ifTrue: [^ self].  "No change"
		"Cursor has moved where its spotShape is visible"
		spotOn := true.
		self spotChanged.
		spotBuffer offset: cp - (spotBuffer extent // 2).
		self spotChanged.
		(BitBlt current toForm: spotBuffer)
			"clear the buffer"
			fill: spotBuffer boundingBox fillColor: (Bitmap with: 0) rule: Form over;
			"Clip anything outside the base form"
			clipRect: (spotBuffer boundingBox
				intersect: (self bounds translateBy: spotBuffer offset negated));
			"Fill the spotBuffer with the spot image"
			copyForm: spotImage to: self position - spotBuffer offset rule: Form over;
			"Mask everything outside the spot shape to 0 (transparent)."
			copyForm: spotShape to: spotShape offset negated rule: Form and
				colorMap: (Bitmap with: 0 with: 16rFFFFFFFF)]
		ifFalse:
		[spotOn ifTrue: [self spotChanged. spotOn := false]]! !

!ImageMorphWithSpotlight methodsFor: 'testing' stamp: 'di 5/24/1999 21:43'!
stepTime

	^ 20! !

Morph subclass: #BackgroundMorph
	instanceVariableNames: 'image offset delta running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

!BackgroundMorph commentStamp: 'StephaneDucasse 8/20/2009 15:52' prior: 0!
This morph incorporates tiling and regular motion with the intent of supporting, eg, panning of endless (toroidal) backgrounds.

The idea is that embedded morphs get displayed at a moving offset relative to my position.  Moreover this display is tiled according to the bounding box of the submorphs (subBounds), as much as necesary to fill the rest of my bounds.

	(BackgroundMorph new addMorph: (ImageMorph new image: Form fromUser)) openInWorld!

!BackgroundMorph class methodsFor: 'as yet unclassified' stamp: 'kfr 8/7/2004 16:10'!
test
	"BackgroundMorph test"
	^(BackgroundMorph new addMorph: (ImageMorph new image: Form fromUser))openInWorld.! !

!BackgroundMorph methodsFor: 'menus' stamp: 'dgd 8/30/2003 20:56'!
addCustomMenuItems: aCustomMenu hand: aHandMorph 
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	running
		ifTrue: [aCustomMenu add: 'stop' translated action: #stopRunning]
		ifFalse: [aCustomMenu add: 'start' translated action: #startRunning]! !

!BackgroundMorph methodsFor: 'accessing' stamp: 'fc 7/24/2004 12:23'!
delta
	^delta! !

!BackgroundMorph methodsFor: 'accessing' stamp: 'fc 7/24/2004 12:24'!
delta: aPoint

	delta := aPoint.! !

!BackgroundMorph methodsFor: 'drawing' stamp: 'ar 6/17/1999 01:06'!
drawOn: aCanvas
	"The tiling is solely determined by bounds, subBounds and offset.
	The extent of display is determined by bounds and the clipRect of the canvas."
	| start d subBnds |
	submorphs isEmpty ifTrue: [^ super drawOn: aCanvas].
	subBnds := self subBounds.
	running ifFalse:
		[super drawOn: aCanvas.
		^ aCanvas fillRectangle: subBnds color: Color lightBlue].
	start := subBnds topLeft + offset - bounds topLeft - (1@1) \\ subBnds extent - subBnds extent + (1@1).
	d := subBnds topLeft - bounds topLeft.
"Sensor redButtonPressed ifTrue: [self halt]."
	start x to: bounds width - 1 by: subBnds width do:
		[:x |
		start y to: bounds height - 1 by: subBnds height do:
			[:y | aCanvas translateBy: (x@y) - d clippingTo: bounds
				during:[:tileCanvas| self drawSubmorphsOn: tileCanvas]]].! !

!BackgroundMorph methodsFor: 'layout' stamp: ''!
fullBounds
	^ self bounds! !

!BackgroundMorph methodsFor: 'drawing' stamp: 'ar 12/30/2001 19:16'!
fullDrawOn: aCanvas
	(aCanvas isVisible: self fullBounds) ifFalse:[^self].
	running ifFalse: [
		^aCanvas clipBy: (bounds translateBy: aCanvas origin)
				during:[:clippedCanvas| super fullDrawOn: clippedCanvas]].
	(aCanvas isVisible: self bounds) ifTrue:[aCanvas drawMorph: self].
! !

!BackgroundMorph methodsFor: 'initialization' stamp: 'dgd 2/14/2003 22:43'!
initialize
	"initialize the state of the receiver"
	super initialize.
""
	offset := 0 @ 0.
	delta := 1 @ 0.
	running := true! !

!BackgroundMorph methodsFor: 'layout' stamp: ''!
layoutChanged
	"Do nothing, since I clip my submorphs"! !

!BackgroundMorph methodsFor: 'accessing' stamp: 'fc 7/24/2004 13:50'!
offset
	^offset! !

!BackgroundMorph methodsFor: 'accessing' stamp: 'fc 7/24/2004 13:50'!
offset: aPoint
	offset := aPoint! !

!BackgroundMorph methodsFor: 'accessing' stamp: ''!
slideBy: inc
	submorphs isEmpty ifTrue: [^ self].
	offset := offset + inc \\ self subBounds extent.
	self changed! !

!BackgroundMorph methodsFor: 'accessing' stamp: ''!
startRunning
	running := true.
	self changed! !

!BackgroundMorph methodsFor: 'stepping and presenter' stamp: 'fc 7/24/2004 13:47'!
step
	
	running ifTrue: [self slideBy: delta]! !

!BackgroundMorph methodsFor: 'testing' stamp: ''!
stepTime
	"Answer the desired time between steps in milliseconds."

	^ 20! !

!BackgroundMorph methodsFor: 'accessing' stamp: ''!
stopRunning
	running := false.
	self changed! !

!BackgroundMorph methodsFor: 'accessing' stamp: 'aoy 2/17/2003 01:20'!
subBounds
	"calculate the submorph bounds"

	| subBounds |
	subBounds := nil.
	self submorphsDo: 
			[:m | 
			subBounds := subBounds isNil
						ifTrue: [m fullBounds]
						ifFalse: [subBounds merge: m fullBounds]].
	^subBounds! !

Morph subclass: #BouncingAtomsMorph
	instanceVariableNames: 'damageReported infectionHistory transmitInfection recentTemperatures temperature t0 flag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MorphExamplesAndDemos'!

BouncingAtomsMorph class
	instanceVariableNames: 'compute'!

!BouncingAtomsMorph commentStamp: '<historical>' prior: 0!
This morph shows how an ideal gas simulation might work. When it gets step messages, it makes all its atom submorphs move along their velocity vectors, bouncing when they hit a wall. It also exercises the Morphic damage reporting and display architecture. Here are some things to try:

  1. Resize this morph as the atoms bounce around.
  2. In an inspector on this morph, evaluate "self addAtoms: 10."
  3. Try setting quickRedraw to false in invalidRect:. This gives the
     default damage reporting and incremental redraw. Try it for
     100 atoms.
  4. In the drawOn: method of AtomMorph, change drawAsRect to true.
  5. Create a HeaterCoolerMorph and embed it in the simulation. Extract
	it and use an inspector on it to evaluate "self velocityDelta: -5", then
     re-embed it. Note the effect on atoms passing over it.
!

BouncingAtomsMorph class
	instanceVariableNames: 'compute'!

!BouncingAtomsMorph class methodsFor: 'parts bin' stamp: 'sw 8/2/2001 01:21'!
descriptionForPartsBin
	^ self partName:	'BouncingAtoms'
		categories:		#('Demo')
		documentation:	'The original, intensively-optimized bouncing-atoms simulation by John Maloney'! !

!BouncingAtomsMorph class methodsFor: 'initialization' stamp: 'asm 4/10/2003 12:57'!
initialize

	self registerInFlapsRegistry.	! !

!BouncingAtomsMorph class methodsFor: 'initialization' stamp: 'asm 4/10/2003 12:58'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: #(BouncingAtomsMorph	new	'Bouncing Atoms'	'Atoms, mate')
						forFlapNamed: 'Widgets']! !

!BouncingAtomsMorph class methodsFor: 'initialization' stamp: 'asm 4/11/2003 12:32'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!BouncingAtomsMorph methodsFor: 'other' stamp: 'CosteDucroz 3/5/2012 16:06'!
addAtoms: n
	"Add a bunch of new atoms."

	| a |
	n timesRepeat: [
		a := CrossMorph     new.
		a randomPositionIn: bounds maxVelocity: 10.
		self addMorph: a].
	self stopStepping.
! !

!BouncingAtomsMorph methodsFor: 'menu' stamp: 'dgd 8/30/2003 21:15'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu add: 'startInfection' translated action: #startInfection.
	aCustomMenu add: 'set atom count' translated action: #setAtomCount.
	aCustomMenu add: 'show infection history' translated action: #showInfectionHistory:.
! !

!BouncingAtomsMorph methodsFor: 'submorphs-add/remove' stamp: 'CosteDucroz 3/5/2012 16:08'!
addMorphFront: aMorph
	"Called by the 'embed' meta action. We want non-atoms to go to the back."
	"Note: A user would not be expected to write this method. However, a sufficiently advanced user (e.g, an e-toy author) might do something equivalent by overridding the drag-n-drop messages when they are implemented."

	(aMorph isMemberOf: CrossMorph   )
		ifTrue: [super addMorphFront: aMorph]
		ifFalse: [super addMorphBack: aMorph].! !

!BouncingAtomsMorph methodsFor: 'drawing' stamp: 'di 1/4/1999 20:22'!
areasRemainingToFill: aRectangle
	color isTranslucent
		ifTrue: [^ Array with: aRectangle]
		ifFalse: [^ aRectangle areasOutside: self bounds]! !

!BouncingAtomsMorph methodsFor: 'other' stamp: 'aoy 2/15/2003 21:38'!
collisionPairs
	"Return a list of pairs of colliding atoms, which are assumed to be
circles of known radius. This version uses the morph's positions--i.e.
the top-left of their bounds rectangles--rather than their centers."

	| count sortedAtoms radius twoRadii radiiSquared collisions p1 continue j p2 distSquared m1 m2 |
	count := submorphs size.
	sortedAtoms := submorphs 
				asSortedCollection: [:mt1 :mt2 | mt1 position x < mt2 position x].
	radius := 8.
	twoRadii := 2 * radius.
	radiiSquared := radius squared * 2.
	collisions := OrderedCollection new.
	1 to: count - 1
		do: 
			[:i | 
			m1 := sortedAtoms at: i.
			p1 := m1 position.
			continue := (j := i + 1) <= count.
			[continue] whileTrue: 
					[m2 := sortedAtoms at: j.
					p2 := m2 position.
					continue := p2 x - p1 x <= twoRadii  
								ifTrue: 
									[distSquared := (p1 x - p2 x) squared + (p1 y - p2 y) squared.
									distSquared < radiiSquared 
										ifTrue: [collisions add: (Array with: m1 with: m2)].
									(j := j + 1) <= count]
								ifFalse: [false]]].
	^collisions! !

!BouncingAtomsMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:14'!
defaultColor
"answer the default color/fill style for the receiver"
	^ Color
		r: 0.8
		g: 1.0
		b: 0.8! !

!BouncingAtomsMorph methodsFor: 'drawing' stamp: 'CosteDucroz 2/15/2012 10:40'!
drawOn: aCanvas
	"Clear the damageReported flag when redrawn."

	super drawOn: aCanvas.
	damageReported := false.
	

	aCanvas fillRectangle: (self bounds topLeft extent: 16@14) color: Color red.
	aCanvas drawString: submorphs size asString at: self bounds topLeft .! !

!BouncingAtomsMorph methodsFor: 'initialization' stamp: 'dgd 3/7/2003 14:14'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	damageReported := false.
	self extent: 400 @ 250.

	infectionHistory := OrderedCollection new.
	transmitInfection := false.
	self addAtoms: 30! !

!BouncingAtomsMorph methodsFor: 'initialization' stamp: 'ar 8/13/2003 11:41'!
intoWorld: aWorld
	"Make sure report damage at least once"
	damageReported := false.
	super intoWorld: aWorld.! !

!BouncingAtomsMorph methodsFor: 'change reporting' stamp: 'ar 11/12/2000 18:42'!
invalidRect: damageRect from: aMorph
	"Try setting 'quickRedraw' to true. This invalidates the entire morph, whose bounds typically subsume all it's submorphs. (However, this code checks that assumption and passes through any damage reports for out-of-bounds submorphs. Note that atoms with super-high velocities do occaisionally shoot through the walls!!) An additional optimization is to only submit only damage report per display cycle by using the damageReported flag, which is reset to false when the morph is drawn."

	| quickRedraw |
	quickRedraw := true.  "false gives the original invalidRect: behavior"
	(quickRedraw and:
	 [(bounds origin <= damageRect topLeft) and:
	 [damageRect bottomRight <= bounds corner]]) ifTrue: [
		"can use quick redraw if damage is within my bounds"
		damageReported ifFalse: [super invalidRect: bounds from: self].  "just report once"
		damageReported := true.
	] ifFalse: [super invalidRect: damageRect from: aMorph].  "ordinary damage report"! !

!BouncingAtomsMorph methodsFor: 'menu' stamp: 'DamienCassou 9/29/2009 09:03'!
setAtomCount

	| countString count |
	countString := UIManager default
		request: 'Number of atoms?' translated
		initialAnswer: self submorphCount printString.
	countString isEmptyOrNil ifTrue: [^ self].
	count := Integer readFrom: (ReadStream on: countString).
	self removeAllMorphs.
	self addAtoms: count.
! !

!BouncingAtomsMorph methodsFor: 'other' stamp: 'jm 6/28/1998 18:31'!
showInfectionHistory: evt
	"Place a graph of the infection history in the world."

	| graph |
	infectionHistory isEmpty ifTrue: [^ self].
	graph := GraphMorph new data: infectionHistory.
	graph extent: ((infectionHistory size + (2 * graph borderWidth) + 5)@(infectionHistory last max: 50)).
	evt hand attachMorph: graph.
! !

!BouncingAtomsMorph methodsFor: 'menu' stamp: ''!
startInfection

	self submorphsDo: [:m | m infected: false].
	self firstSubmorph infected: true.
	infectionHistory := OrderedCollection new: 500.
	transmitInfection := true.
	self startStepping.
! !

!BouncingAtomsMorph methodsFor: 'stepping and presenter' stamp: 'CosteDucroz 3/5/2012 16:56'!
step
	"Bounce those atoms!!"

	| r bounces size |
	super step.
	bounces := 0.
	r := bounds origin corner: (bounds corner - (8@8)).
	self submorphsDo: [ :m |
		(m isMemberOf: CrossMorph   ) ifTrue: [
			(m bounceIn: r) ifTrue: [bounces := bounces + 1]]].
	"compute a 'temperature' that is proportional to the number of bounces
	 divided by the circumference of the enclosing rectangle"
	self updateTemperature: (100000.0 * bounces) / (r width + r height).
	transmitInfection ifTrue: [self transmitInfection]. 
	
	t0 ifNil:  [t0:= DateAndTime now].
	flag ifNil: [flag=1].
	
	((DateAndTime now - t0 ) asSeconds >=1)
		ifTrue: [
				(submorphs size>=30)
					ifTrue:[flag:=0.].
					
				(submorphs size<=1)
					ifTrue:[flag:=1.].
					
				self color: Color random.
				

				t0:=DateAndTime now.
				
				(flag=1)
					ifTrue:[self addAtoms: 1.].
				(flag=0)
					ifTrue:[self removeMorph: submorphs atRandom .].
					
				submorphs atRandom width: submorphs size.
				submorphs atRandom height: submorphs size.
					
				self startStepping.].
			
				submorphs atRandom color: Color random.! !

!BouncingAtomsMorph methodsFor: 'testing' stamp: 'CosteDucroz 2/14/2012 14:39'!
stepTime
	"As fast as possible."

	^ 0
! !

!BouncingAtomsMorph methodsFor: 'other' stamp: 'jm 6/28/1998 18:20'!
transmitInfection

	| infected count |
	self collisionPairs do: [:pair |
		infected := false.
		pair do: [:atom | atom infected ifTrue: [infected := true]].
		infected
			ifTrue: [pair do: [:atom | atom infected: true]]].

	count := 0.
	self submorphsDo: [:m | m infected ifTrue: [count := count + 1]].
	infectionHistory addLast: count.
	count = submorphs size ifTrue: [
		transmitInfection := false.
		self stopStepping].
! !

!BouncingAtomsMorph methodsFor: 'other' stamp: 'dgd 2/22/2003 13:36'!
updateTemperature: currentTemperature 
	"Record the current temperature, which is taken to be the number of atoms that have bounced in the last cycle. To avoid too much jitter in the reading, the last several readings are averaged."

	recentTemperatures isNil 
		ifTrue: 
			[recentTemperatures := OrderedCollection new.
			20 timesRepeat: [recentTemperatures add: 0]].
	recentTemperatures removeLast.
	recentTemperatures addFirst: currentTemperature.
	temperature := recentTemperatures sum asFloat / recentTemperatures size! !
ClockMorph initialize!
StickyPadMorph initialize!
StarMorph initialize!
MovingEyeMorph initialize!
AnimatedImageMorph initialize!
BouncingAtomsMorph initialize!
FrameRateMorph initialize!
